/*	$Id: cnode.h,v 1.25 2005/08/26 00:48:14 phil Exp $ */
/* 
 * Mach Operating System
 * Copyright (c) 1990 Carnegie-Mellon University
 * Copyright (c) 1989 Carnegie-Mellon University
 * All rights reserved.  The CMU software License Agreement specifies
 * the terms and conditions for use and redistribution.
 */

/*
 * This code was written for the Coda file system at Carnegie Mellon University.
 * Contributers include David Steere, James Kistler, and M. Satyanarayanan.
 * Modifications for Linux port by Peter Braam
 */

/* Rewritten for NT by Peter Braam and Phil Nelson. */

#ifndef	_CNODE_H_
#define	_CNODE_H_

#include <ntddk.h>

#define KERNEL 1

typedef struct coda_filehandle coda_filehandle;

typedef struct coda_inode {
    int			    c_magic;    // Magic number!
    struct dllist_chain	    c_alllist; 
    struct dllist_chain	    c_fidlist;
    struct dllist_chain	    c_namelist;
    struct dllist_chain	    c_freelist; 
    ERESOURCE		    c_lock;     // For locks
    // Reused inodes will be zeroed from here on
    coda_filehandle *	    c_fh_list;  // list of filehandles  
    int                     c_fh_cnt;   // number of file handles
    CodaFid		    c_fid;
    int			    c_count;	// count of inodes below this one
    struct coda_ntvolume *  c_ntvol;
    struct coda_inode *	    c_parent;
    int			    c_status;	// status flags, see below
    char		    c_name[CODA_MAXNAMLEN];
    FS_FILE_ATTRIBUTES      c_fattr;
    ULONG                   c_type;	//
    int			    c_open_cnt; // Count the number of opens.
    char		    c_containername[CODA_MAXNAMLEN]; // File name
    PFILE_OBJECT            c_object;   // NT file object
    HANDLE *		    c_handle;	// NT file handle
    KEVENT 		    c_event;    // For the work queues 
    PIO_WORKITEM	    c_work_item; // Needed to free it
} coda_inode ;


//
// filehandle support!
//
//  uid stored here for opens() inside read/writes.  Thread/process
//  appear to not be the correct user process.
//

typedef struct coda_filehandle {
    int		fh_unique;
    coda_inode *fh_inode;
    coda_inode *fh_dir;
    int		fh_isopen;
    ULONG       fh_granted; 
    ULONG       fh_lastacc; 
    int		fh_openflags;
    int         fh_uid;	
    struct coda_filehandle *fh_next;	// for lists ... ?? needed
} coda_filehandle;


typedef struct coda_ntvolume {
    int			v_status;
    int			v_refcount;
    coda_inode *	v_rootinode;
    coda_filehandle *   v_rootfh;
    coda_inode *        v_ctlinode;
    coda_filehandle *   v_ctlfh;
    PWCHAR		v_name;
    int			v_namelen;
} coda_ntvolume;

#define V_MOUNTED 	0x1
#define V_ROOT		0x2
#define CODA_MOUNTED(v) ((v)->v_status & V_MOUNTED)
#define CODA_ROOTVOL(v) ((v)->v_status & V_ROOT)

extern coda_ntvolume coda_nt_volume;


// XXX -- change which struct the MAGIC goes into if fh struct works
// out better!
#define CODA_MAGIC 0X4711

#define CODA_NTVOL(v)  ((v) == &coda_nt_volume)
#define CODA_ISOURS(f) ((f) && ((f)->c_magic == CODA_MAGIC) && CODA_NTVOL(f->c_ntvol))
#define CODA_ISDIR(f) (CODA_ISOURS((f)) && ((f)->c_type & C_VDIR))

void inode_init();
void inode_freeall();
coda_inode *inode_find_fid(CodaFid *);
coda_inode *inode_find_name(coda_inode *parent, char *name, int CaseInsensitive);
coda_inode *inode_new(CodaFid *, coda_inode *parent, char* name);
void inode_free(coda_inode *);
void inode_pin(coda_inode *);
void inode_unpin(coda_inode *);

void inode_cache_delete (coda_inode *inode);
void inode_release(coda_inode *inode);
void inode_hash_insert(coda_inode *inode);
void inode_rename(coda_inode *inode, coda_inode *parent, char *name);

void inode_close_handle(coda_inode *inode); 

int coda_get_fileobject(coda_filehandle *, int);

// filehandle routines
coda_filehandle *get_cfh (coda_inode *inode, coda_inode *dir);
void free_cfh (coda_filehandle *ofh);

// status
#define I_REL    0x010 // inode has been released
#define I_VATTR  0x020 // attributes valid
#define I_PINNED 0x040 // inode is pinned
#define I_PURGE  0x080 // call inode_release when unpinned.
#define I_DELETE 0x100 // File is Deleted, call venus close to effect delete.
#define I_LOCKED 0x200 // inode is locked by following macros
#define I_ZAPPED 0x400 // set if inode is locked with zapfile 

// Don't ever try to lock an Inode while you have the wrapper lock locked.
// You may GrabLock the wrapper lock while an Inode is locked.

#define LockInode(Inode)\
   { CDEBUG((D_MISC, "LockInode: inode %s (IRQL=%d)\n",\
	     coda_f2s(&(Inode->c_fid)), KeGetCurrentIrql()));\
     ExAcquireResourceExclusiveLite(&Inode->c_lock, TRUE);\
     GrabLock(); Inode->c_status |= I_LOCKED; DropLock();\
   }

#define UnLockInode(Inode)\
   { \
     CDEBUG((D_MISC, "UnLockInode: IRQL %d\n", KeGetCurrentIrql()));\
     ExReleaseResourceLite(&Inode->c_lock);\
     CDEBUG((D_MISC, "UnLockInode: inode %s\n",\
	     coda_f2s(&(Inode->c_fid))));\
     GrabLock(); Inode->c_status &= ~I_LOCKED; DropLock();\
   }


// Seting and clearing inode flags. 
#define inode_setflag(inode, flag) \
	GrabLock(); (inode)->c_status |= (flag); DropLock()
#define inode_unsetflag(inode, flag) \
	GrabLock(); (inode)->c_status &= ~(flag); DropLock()
#define inode_hasflag(inode, flag) ((inode)->c_status & (flag))

// Locking the Cache

#define LockCache() 
//	ExAcquireResourceExclusiveLite(&cache_lock, TRUE)

#define UnLockCache()
//	ExReleaseResourceLite(&cache_lock)

// Debug .. print cache.

void print_inode_cache();

#endif	not _CNODE_H_

